﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows;
using System.Collections;

namespace UndoOperations
{
    #region ListUndeleteStrategy
    // This sample strategy assumes that the objects that were removed from the collection 
    // can be reinserted into the collection and therefore this does not work with things 
    // like dataview, datatable and dataset. you should refer to the advanced undo sample 
    // in the samples browser for more examples

    public class ListUndeleteStrategy : UndeleteRecordsStrategy
    {
        public ListUndeleteStrategy(IList<Record> records)
        {
        }

        public override bool CanUndelete(IList<UndeleteRecordsStrategy.RecordInfo> oldRecords)
        {
            return true;
        }

        public override IDictionary<RecordInfo, object> Undelete(IList<RecordInfo> oldRecords)
        {
            Dictionary<RecordInfo, object> oldToNewMapping = new Dictionary<RecordInfo, object>();

            RecordInfo[] records = oldRecords.ToArray();
            Comparison<RecordInfo> comparison = delegate(RecordInfo item1, RecordInfo item2)
            {
                return item1.DataItemIndex.CompareTo(item2.DataItemIndex);
            };

            // sort by the original index to ensure we get them in the original order
            Utilities.SortMergeGeneric(records, Utilities.CreateComparer(comparison));

            foreach (RecordInfo record in records)
            {
                RecordManager rm = record.RecordManager;
                IList list = rm.SourceItems as IList;

                if (list == null)
                    continue;

                int newIndex = Math.Min(list.Count, record.DataItemIndex);
                object newDataItem = record.DataItem;

                list.Insert(newIndex, newDataItem);

                // since we're reusing the same object the old and new will be the same
                oldToNewMapping[record] = newDataItem;
            }

            return oldToNewMapping;
        }
    }

    #endregion //ListUndeleteStrategy
}
